using System.Collections;
using System;
using System.Collections.Generic;

using UnityEngine;
using UnityEngine.InputSystem;

using BigSock.Service;

namespace BigSock {

	/*
		Charged projectile attack for the player..
	*/
	public class ChargeAttack : BaseAttack {
		//protected static readonly GameObject PROJECTILE_BASE = new AttackMovement();
		public const string PROJECTILE_NAME = "bullets/chargedbullet";

		
		public override ulong Id => 103;
		public override string Name => "Charged Ball";
		public override string Description => "It's a charged attack that explodes on impact.";

		public ChargeAttack() {
			/*AttackStats = new AttackStats{
				Damage = 4f,
				Knockback = 5f,
				Range = 5f,
				ProjectileSpeed = 3f,
				AttackSpeed = 1f,
				CritChance = 0.25f,
				CritDamageModifier = 3f,
				DamageVariance = 0.5f,
			};*/

			Cooldown = new TimeSpan(0, 0, 0, 2, 0);
			ManaCost = 5;
		}

        public AttackStats CreateAttackStats(float chargeTime) {
            return new AttackStats{
				Damage = 4f * chargeTime,
				Knockback = 5f,
				Range = 5f,
				ProjectileSpeed = 3f,
				AttackSpeed = 1f,
				CritChance = 0.25f,
				CritDamageModifier = 3f,
				DamageVariance = 0.5f,
			};
        }



		/*
			Activates the ability.
			Returns true if the ability was successfully activated. 
				- Even if nothing was hit, used to indicate that cooldowns should be updated.
			This should be overridden in sub-classes for the actual abilities.
		*/
		protected override bool Activate(Character actor, float chargeTime, Vector3? target) {
			if(target == null) return false;

            AttackStats = CreateAttackStats(chargeTime);
			var attack = (AttackStats) AttackStats.Calculate(actor.Stats);
			//var attack = (AttackStats) AttackStats.Apply(actor.Stats);
			attack.Actor = actor;


			var temp = (target.Value - actor.transform.position);
			temp.z = 0;

			var bullet = PrefabService.SINGLETON.Instance(PROJECTILE_NAME, actor.transform.position);
			var bulletScript = bullet.GetComponent<ChargedAttackMovement>();
			//bulletScript.Actor = actor;
			bulletScript.Stats = attack;
			bulletScript.Direction = temp.normalized;


			//MonoBehaviour.Instantiate(PROJECTILE_BASE, (Vector3) actor.transform.position, PROJECTILE_BASE.transform.rotation);
			return true;
		}

	}

}