using System.Collections;
using System;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.InputSystem;


namespace BigSock.Item {

	/*
		An inventory of a character.
	*/
	public class Inventory {
		/*
			The items in the inventory.
		*/
		public List<IItem> Items { get; protected set; } = new List<IItem>();

		/*
			The modifier from all the passives.
		*/
		public ICharacterStats Modifier { get; protected set; } = new CharacterStats();


		/*
			The max number of items the inventory can hold.
		*/
		public int Cap { get; set; } = 10;


		/*
			The max number of items the inventory can hold.
		*/
		public Character Owner { get; }


		public Inventory(Character owner) {
			if(owner == null) throw new ArgumentNullException(paramName: nameof(owner));
			Owner = owner;
		}


		/*
			Adds an item to the inventory and manages changes.
		*/
		public bool AddItem(IItem item) {
			if(Items.Count >= Cap) return false;
			
			Items.Add(item);

			// Add the passive effects to the modifier.
			if(item is PassiveItemBase passive) {
				Modifier = Modifier.Add(passive.Modifier);
			}

			// Add the listener for this item to the owner.
			else if(item is ConditionalItemBase conditional) {
				Owner.AddItemListener(conditional);
			}
			//! Add ifs to handle the other 2 types of items.

			return true;
		}

		/*
			Removes an item from the inventory and manages changes.
		*/
		public bool RemoveItem(IItem item) {
			if(!Items.Remove(item)) return false;
			
			// Remove the passive effects from the modifier.
			if(item is PassiveItemBase passive) {
				Modifier = Modifier.Remove(passive.Modifier);
			}

			// Remove the listener for this item from the owner.
			else if(item is ConditionalItemBase conditional) {
				Owner.RemoveItemListener(conditional);
			}
			//! Add ifs to handle the other 2 types of items.

			return true;
		}

	}
}